/* 
    eprimer_v051.js
 
    Purpose: BrainVoyager tool to convert an E-Prime Log file (*.txt) to a BrainVoyager  stimulation protocol file.
	Installation: place the *.dll/dylib/so, *.js and *.ui files in /(My) Documents/BrainVoyager/BVExtensions/Plugins/.
	Activate: in BrainVoyager, go to the BrainVoyager "Plugins..." menu option and select "eprimer". 
	Updates:
        v0.5.1 (July 2021):
                - update to BrainVoyager 21.4+
                - add pathname to protocolname (in function generate_stimulation_protocol_from_EPrimeLog())
                - using split() functions to extract names (in function readparametersfile())
		v0.5: update to BrainVoyager QX 2.3
		v0.4: Save condition parameters to text file and load for other E-Prime *.txt file
		
	Current limitations: 
	- only msec resolution
    - configuration of conditions only via parameters text file
		
 Hester Breman, Brain Innovation B.V., advice by Dr. Tobias Otto, Maastricht University; 2008-2021.
*/


var scriptObj = new Object;
scriptObj.scriptNameForUser = "eprimer_v051";
scriptObj.dialogFileName    = "eprimer_v051.ui";
scriptObj.dialogAccessName  = "eprimerdlg";

// Initializing global variables for GUI elements
var dlg;
// import widgets
var lblGetLogFile, lblGetConfigFile, btnGetLogFile, btnGetConfigFile;
// configuration widgets
var rbtnManualConfig, rbtnAutoConfig, leConfigFileName;
// misc widgets
var btnConvertLog2Prt;


////////////////////////////////////////////////////////////////////////////////////////////

var msecresolution; // boolean variable indicating stimulation protocol resolution

/*~~~~~~~~~~~~ Experiment object ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

// constructor of experiment object
function Experimentobject(nrofconditions)  { 
	
    // member variables
	var _nrofconditions;
	var _experimentname;
	var _variablenamearray; // (n) conditions as found in E-Prime Log file
	var _conditionnamearray; // (n) renamed by user
	var _colorindexarray; // (n x 3)
	var _condbeginvararray;  // new per v0.2
	var _condbeginproparray; 
	var _condendvararray; 
	var _condendproparray; // until here
	var _absolutezero;
	var _begininterval;
	var _endinterval;
	
	
	 	this._nrofconditions = nrofconditions;
		this._variablenamearray = new Array(nrofconditions);
		this._conditionnamearray = new Array(nrofconditions);
		this._colorindexarray = new Array(nrofconditions);
		this._condbeginvararray = new Array(nrofconditions);  // new per v0.2
		this._condbeginproparray = new Array(nrofconditions); 
		this._condendvararray = new Array(nrofconditions);  
		this._condendproparray = new Array(nrofconditions); // until here		
	
    // declare member functions
    this.getnrofconditions = getnrofconditions;
    this.setexperimentname = setexperimentname;
    this.getexperimentname = getexperimentname;
    this.setvariables = setvariables;
    this.getvariables = getvariables;
    this.setconditionnames = setconditionnames;
    this.getconditionnames = getconditionnames;
    this.getconditionname = getconditionname;
    this.setcolorindexarray = setcolorindexarray;
    this.getcolorindexarray = getcolorindexarray;
    this.getcolor = getcolor;
    this.setzero = setzero;
    this.getzero = getzero;
    this.setbegininterval = setbegininterval;
    this.setendinterval = setendinterval;
    this.setconditionsbegin = setconditionsbegin;
    this.setconditionsend = setconditionsend;
    this.getconditionsvarbegin = getconditionsvarbegin;
    this.getconditionvarbegin = getconditionvarbegin;
    this.getconditionspropbegin = getconditionspropbegin;
    this.getconditionpropbegin = getconditionpropbegin;
    this.getconditionsvarend = getconditionsvarend;
    this.getconditionvarend = getconditionvarend;
    this.getconditionspropend = getconditionspropend;
    this.getconditionpropend = getconditionpropend;
    this.printtolog = printtolog;                

}
	
function getnrofconditions() {
    return this._nrofconditions;
}

function setexperimentname(experimentname) {
    this._experimentname = experimentname;
}

function getexperimentname() {
    return this._experimentname;
}

function setvariables(variableslist) {
    this._variablenamearray = variableslist;
}

function getvariables() {
    return this._variablenamearray;
}

function setconditionnames(newnameslist) {
    this._conditionnamearray = newnameslist;
}

function getconditionnames() {
    return this._conditionnamearray;
}

function getconditionname(index) {
    return this._conditionnamearray[index];
}
    
function setcolorindexarray(colorindiceslist) {
    this._colorindexarray = colorindiceslist;
}

function getcolorindexarray() {
    return this._colorindexarray;
}

function getcolor(index) {
    return this._colorindexarray[index];
}

function setzero(zero) {
    this._absolutezero = zero;
}

function getzero() {
    return this._absolutezero;
}
    
function setbegininterval(begin) {
    this._begininterval = begin;
}

function setendinterval(end) {
    this._endinterval = end;
}

function setconditionsbegin(beginvarlist, beginproplist) {
    this._condbeginvararray = beginvarlist;
    this._condbeginproparray = beginproplist;
}

function setconditionsend(endvarlist, endproplist) {
    this._condendvararray = endvarlist;
    this._condendproparray = endproplist;
}

function getconditionsvarbegin() {
    return this._condbeginvararray;
}

function getconditionvarbegin(index) {
    return this._condbeginvararray[index];
}

function getconditionspropbegin() {
    return this._condbeginproparray;
}

function getconditionpropbegin(index) {
    return this._condbeginproparray[index];
}

function getconditionsvarend() {
    return this._condendvararray;
}

function getconditionvarend(index) {
    return this._condendvararray[index];
}

function getconditionspropend() {
    return this._condendproparray;
}

function getconditionpropend(index) {
    return this._condendproparray[index];
}

function printtolog() {

    var txtstring = "Experimentname: " + this._experimentname + "\n" +
                    "Number of conditions: " + this._nrofconditions + "\n" +
                    "Variable defining zero: " + this._absolutezero;
    BrainVoyager.PrintToLog(txtstring);
    var i;
    for (i = 0; i < this._nrofconditions; i++) {
        BrainVoyager.PrintToLog("Condition: " + this._variablenamearray[i]);
        BrainVoyager.PrintToLog("New name: " + this._conditionnamearray[i]);
        BrainVoyager.PrintToLog("Color: " + getcolorname(this._colorindexarray[i]));
        var intbeginname = this._condbeginvararray[i] + "." + this._condbeginproparray[i];
        var intendname = this._condendvararray[i] + "." + this._condendproparray[i];		
        BrainVoyager.PrintToLog("Begin of interval: " + intbeginname);
        BrainVoyager.PrintToLog("End of interval: " + intendname);
    }
}

/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

// main function
scriptObj.generate_stimulation_protocol_from_EPrimeLog = function() {
	
	// declarations
	var cancel = false;
	
	// 0. prepare
    var expobj;
    var logvaluesarray, variablesarray, conditionnamesarray;
	var conditionsobjarray = new Array();
	BrainVoyager.ShowLogTab();
	BrainVoyager.PrintToLog("Start conversion...");
    
    var logname = lblGetLogFile.text;
    var parametersfilename = lblGetConfigFile.text;  
    BrainVoyager.PrintToLog("Log file: " + logname);
    BrainVoyager.PrintToLog("Parameters file: " + parametersfilename);
    this.eprimerdlg.close();  
    
    if ((logname.length == 0) || (parametersfilename.length == 0)) { 
		cancel = true;
	} else {	

        // 1. read the files        
        logvaluesarray = readeprimelogfile(logname); // read E-Prime *.txt 
        variablesarray = extractvariables(logvaluesarray);// keep only lines with variables 
        conditionnamesarray = getconditions(variablesarray);
        printarraytolog("Variable names:", conditionnamesarray);
		
		// 2+3: new in version 0.4: let user configure conditions or read conditions from text file
		var expobj;
		expobj = readparametersfile(parametersfilename);
		expobj.printtolog();
	
		// 4. collect start and endtimes
		if (!cancel) { 
			var success = prepareprotocol(expobj.getexperimentname());
			if (success) {
				success = createconditions(expobj);
				if (success) addintervals_v02(logvaluesarray, expobj); // new version 02: separate begin and end names per condition
	
				// 5. save BrainVoyager stimulation protocol
                var paramfile = new QFileInfo(parametersfilename);// not QFile?
            //    var protocolname = paramfile.absoluteFilePath() + "/" + expobj.getexperimentname();
                var protocolname = paramfile.dir().path() + "/" + expobj.getexperimentname();
                BrainVoyager.PrintToLog("Protocolname: " + protocolname);
				if (success) success = saveprotocol(protocolname);
			}
		}
	}
    
}

/* Only keep the lines that contain information with a variable */
// (from eprimelogfunctions.qs)

function extractvariables(logvaluesarray) {
	
    BrainVoyager.PrintToLog("Extracting variables...");
	var linecounter;
	var conditionarray = new Array();
	var nroflines = logvaluesarray.length;
	for (linecounter = 0; linecounter < nroflines; linecounter++) {
		tmpline = logvaluesarray[linecounter];
		if (containsconditionpart(tmpline)) {
				conditionarray.push(tmpline);
		}		
	}	
	return conditionarray;
}


// Return a list with conditionnames 
function getconditions(variablearray) {
	
    BrainVoyager.PrintToLog("Get conditions...");
	var conditionnamearray = new Array(); 
	var counter, counter2, words, fragments;
	for (counter = 0; counter < variablearray.length; counter++) {
	
		    // now first obtain the left part of the sentence 
			words = variablearray[counter].split('.');
			if (conditionnamearray.toString().indexOf(words[0]) == -1) {
					
				// now get the word without the empty space
				fragments = words[0].split('\t');
				for (counter2 = 0; counter2 < fragments.length; counter2++) {
					if (!fragments[counter2].length == 0) {
						if (conditionnamearray.toString().indexOf(fragments[counter2]) == -1) {
							conditionnamearray.push(fragments[counter2]);
						}
					} // end if (check for empty)
				} // end for
			} // end if (check left part of text fragment)
	} // end counting through all variable names
	return conditionnamearray;
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/* 
  eprimelogfunctions.qs
  
  05-03-2009: added extracttimingparameters(), isintervalstart(), isintervalend()
  01-02-2008: created.
  HB, BI.
*/  
 
function isconditionpart(name) {
		
	var isconditionpart;
	if ((name == "OnsetDelay") || (name == "OnsetTime") || (name == "DurationError") || (name == "RTTime") ||
		(name == "StartTime") || (name == "FinishTime") || (name == "OffsetTime") || (name == "StartTime") ||
		(name == "ACC") || (name == "RT") || (name == "RESP") || (name == "CRESP")) {
		isconditionpart = true;
	} else {
		isconditionpart = false;
	}
	return isconditionpart;
}

function containsconditionpart(line) {
	
	var containsconditionpart;
	if ((line.indexOf("OnsetDelay") > -1) || (line.indexOf("OnsetTime") > -1) || (line.indexOf("DurationError") > -1) || (line.indexOf("RTTime") > -1) ||
		 (line.indexOf("StartTime") > -1) ||  (line.indexOf("FinishTime") > -1) || (line.indexOf("OffsetTime") > -1) ||
		(line.indexOf("ACC") > -1) || (line.indexOf("RT") > -1) || (line.indexOf("RESP") > -1) || (line.indexOf("CRESP") > -1)) {
		containsconditionpart = true;
	} else {
		containsconditionpart = false;
	}
	return containsconditionpart;	
}

function conditionexists(name, conditionsobjarray) {
	
	var counter;
	var conditionexists = -1;
	for (counter = 0; counter < conditionsobjarray.length; counter++) {
		if (conditionsobjarray[counter].name == name) {
			conditionexists = counter;
			break;
		}	
	}
	return conditionexists;
}
	
function isintervalstart(line) {
	
	var isstart = false;
	if (line.indexOf("OnsetTime") > -1) {
		isstart = true;
	} 
	return isstart;
}

function isintervalend(line) {
	
	var isend = false;
	if (line.indexOf("RT") > -1) {
		isend = true;
	} 
	return isend;
}


// loop through the log values and add in protocol
function extracttimingparameters(conditionsarray, logvaluesarray, timingpropsarray) {
	
	var linecounter;
	var timingarray = new Array();
	var nroflines = logvaluesarray.length;
	var isvariable = false;	
	var absolutezero = timingpropsarray[0];
	var begininterval = timingpropsarray[1];
	var endinterval = timingpropsarray[2];
	for (linecounter = 0; linecounter < nroflines; linecounter++) {
	
		line = logvaluesarray[linecounter];
		if (isintervalstart(line)) {
			
			if (line.indexOf(begininterval) > -1) {
			// save in experimentobject or add directly to BrainVoyager stimulation protocol
				
				// now get the word without the empty space
				fragments = line.split('\t');
				for (counter = 0; counter < fragments.length; counter++) {
					
					if (!fragments[counter].isEmpty()) {
						
						// check whether start of interval
						if (conditionnamearray.toString().indexOf(fragments[counter]) == -1) {
							conditionnamearray.push(fragments[counter]);
						// check whether end of interval	
						} else if (conditionnamearray.toString().indexOf(fragments[counter]) == -1) {
							conditionnamearray.push(fragments[counter]);
						// 
						} else if (conditionnamearray.toString().indexOf(fragments[counter]) == -1) {
							conditionnamearray.push(fragments[counter]);
						}
							
					} // end if (check for empty)
				} // end for						
				
			}
		} else if (line.indexOf(endinterval) > -1) {
		
			// save in object or add directly to BrainVoyager stimulation protocol
		}
	}
//	return experimentobject or success
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/*
  bvqxprotocolfunctions.qs
 
  24-03-2009: added addintervals_v02()
  01-02-2009: created 
  
  HB, BI.
*/

const RES_VOLUMES = 1;
const RES_MSEC = 2;
const TR = 2000;

// clear protocol, set resolution and experiment name
function prepareprotocol(experimentname) {
	
	var success = true;
	BrainVoyager.PrintToLog("Start creating stimulation protocol...");
// => update syntax
	var doc = BrainVoyager.ActiveDocument;
	if (doc != undefined) {
		doc.ClearStimulationProtocol();
		doc.StimulationProtocolExperimentName = experimentname;
//		var doMsecButton = MessageBox.information("Protocol in milliseconds (Yes) or volumes (No)?", MessageBox.Yes, MessageBox.No);
	//	var doMsec = ((doMsecButton == MessageBox.Yes) ? true : false);
		var doMsec = true;
		doc.StimulationProtocolResolution = (doMsec ? RES_MSEC : RES_VOLUMES);
		msecresolution = doMsec;
	} else {
		BrainVoyager.PrintToLog("Could not find a currently opened BrainVoyager project (FMR/VMR).");
		success = false;
	} 
	return success;
}


// create conditions with names and colors
function createconditions(experimentobject) {
	
	var counter, name, color;
	var success = true;
    BrainVoyager.PrintToLog("Start creating conditions...");
	var doc = BrainVoyager.ActiveDocument;
	if (doc != undefined) {
		for (counter = 0; counter < experimentobject.getnrofconditions(); counter++) {      
			name = experimentobject.getconditionname(counter);         
			color = experimentobject.getcolor(counter);        
			doc.AddCondition(name);	 
			doc.SetConditionColor(name, color[0], color[1], color[2]);
		}
	} else {
		BrainVoyager.PrintToLog("Could not find a currently opened BrainVoyager project (FMR/VMR).");
		success = false;
	} 
	return success;
}

function addintervals_v02(logvaluesarray, experimentobject) {
	
	var linecounter, ccounter, currentcondition, fragments, intbegin, intend, len; 
	var foundbegin = false;
	var foundend = false;
		
	var absolutezero = experimentobject.getzero();//_absolutezero;
	var intbeginname; 
	var intendname; 
	var doc = BrainVoyager.ActiveDocument;
	
	// for each condition, loop through the logvalues array to find begin and end intervals
	for (ccounter = 0; ccounter < experimentobject.getnrofconditions(); ccounter++) {
		
		// get current condition name and variable name plus property defining begin and end of condition
		currentcondition = experimentobject.getconditionname(ccounter);
		intbeginname = experimentobject.getconditionvarbegin(ccounter) + "." + experimentobject.getconditionpropbegin(ccounter);
		intendname = experimentobject.getconditionvarend(ccounter) + "." + experimentobject.getconditionpropend(ccounter);		
		
		BrainVoyager.PrintToLog("Searching for condition: " + currentcondition);
		BrainVoyager.PrintToLog("Begin: " + intbeginname);
		BrainVoyager.PrintToLog("End: " + intendname);
		
		// loop over Presentation Log file
		for (linecounter = 0; linecounter < logvaluesarray.length; linecounter++) {
	
			line = logvaluesarray[linecounter];
							
				var j = 0;
				// search for begin
				if (line.indexOf(intbeginname) > -1) {
					
					// extract begin (absolute w.r.t. zeropoint)
					fragments = line.split(' ');
					len = fragments.length;
					intbegin = parseInt(fragments[1]) - absolutezero; // assume that number is last fragment				
					foundbegin = true;
	
				} else if ((line.indexOf(intendname) > -1) && (foundbegin)) {
				
					// extract end (relative w.r.t. interval begin)
					fragments = line.split(' ');
					len = fragments.length;
					intend = parseInt(fragments[1]) - absolutezero; // assume that number is last fragment				
					foundend = true;					
					
				}
				
				// now add interval to condition
				if (foundbegin && foundend) {
					BrainVoyager.PrintToLog("Adding '" +  experimentobject.getconditionname(ccounter) + "': " + intbegin + "-" + intend);
					doc.AddInterval(experimentobject.getconditionname(ccounter), intbegin, intend);
					foundbegin = false;
					foundend = false;
				}
				// (maybe add check on foundend if not foundbegin)
		} // end linecounter
	} // end ccounter
//	return experimentobject or success
}


function saveprotocol(experimentname) {
	
	var success = true;
	var prtname;
	var doc = BrainVoyager.ActiveDocument;
	if (doc != undefined) {
		prtname = experimentname + ".prt";
		doc.SaveStimulationProtocol(prtname);
		
	} else {
		success = false;
	}
	if (success) {
		BrainVoyager.PrintToLog("Saved stimulation protocol: " + prtname + ".");
	} else {
		BrainVoyager.PrintToLog("Could not save stimulation protocol.");
	}
}



/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/*
  colorfunctions.qs
 
  12-03-2009: added more colors
  01-02-2009: created 
  
   HB, BI.
*/

const COLOR_GRAY 		= [128, 128, 128];
const COLOR_BLACK 		= [0, 0, 0];
const COLOR_RED 		= [255, 0, 0];
const COLOR_GREEN 		= [0, 255, 0];
const COLOR_BLUE		= [0, 0, 255];
const COLOR_WHITE 		= [255, 255, 255];
const COLOR_YELLOW 		= [255, 255, 0];
const COLOR_PINK 		= [255, 0, 255];
const COLOR_LIGHTBLUE 	= [0, 255, 255];
const COLOR_BURGUNDY 	= [85,0,0];
const COLOR_DARKGREEN	= [0, 85, 0];
const COLOR_DARKBLUE	= [0,0,85];
const COLOR_ORANGE		= [255, 85, 0];
const COLOR_BRIGHTBLUE	= [0, 85, 255];
const COLOR_PINKRED		= [255,0,85];

const colornames= ["Gray", "Black", "Red", "Green", "Blue", "White", "Yellow", "Pink", "Lightblue", "Burgundy", 
				   "Darkgreen", "Darkblue", "Orange", "Brightblue", "Pinkred"];

// is this function used at all?
function printcolor(color) {
	
	var colorname = Input.getItem("Please select a color", colornames);
	var color = getcolorindex(colorname);
	BrainVoyager.ShowLogTab();
	BrainVoyager.PrintToLog("Color: " + color[0] + color[1] + color[2]);
	
}

function getcolorindex(colorname) {
	
	switch (colorname) {
	
	case "Gray": 		return COLOR_GRAY;
	case "Black": 		return COLOR_BLACK;
	case "Red": 		return COLOR_RED;
	case "Green": 		return COLOR_GREEN;
	case "Blue": 		return COLOR_BLUE;
	case "White": 		return COLOR_WHITE;
	case "Yellow": 		return COLOR_YELLOW;
	case "Pink": 		return COLOR_PINK; 
	case "Lightblue": 	return COLOR_LIGHTBLUE; 
	case "Burgundy": 	return COLOR_BURGUNDY; 				   
	case "Darkgreen": 	return COLOR_DARKGREEN; 
	case "Darkblue": 	return COLOR_DARKBLUE; 
	case "Orange": 		return COLOR_ORANGE; 
	case "Brightblue": 	return COLOR_BRIGHTBLUE; 
	case "Pinkred": 	return COLOR_PINKRED;	

	}
}

function getcolorname(colorindex) {
	
	switch (colorindex) {
	
	case COLOR_GRAY: 		return "Gray";
	case COLOR_BLACK: 		return "Black";
	case COLOR_RED: 		return "Red";
	case COLOR_GREEN:		return "Green";
	case COLOR_BLUE:		return "Blue";
	case COLOR_WHITE:		return "White"; 
	case COLOR_YELLOW: 		return "Yellow";
	case COLOR_PINK:		return "Pink"; 		
	case COLOR_LIGHTBLUE: 	return "Lightblue"; 
	case COLOR_BURGUNDY: 	return "Burgundy"; 
	case COLOR_DARKGREEN: 	return "Darkgreen"; 	
	case COLOR_DARKBLUE: 	return "Darkblue"; 	
	case COLOR_ORANGE:	 	return "Orange"; 		
	case COLOR_BRIGHTBLUE:	return "Brightblue"; 	
	case COLOR_PINKRED:		return "Pinkred"; 	
	}
} 

function getcolornames() {
	return colornames;
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/* 
  functions.qs
  
  10-03-09: added printarraytolog(msg, vararray)
  01-02-09: created
  HB, BI.
*/


function printarraytolog(msg, vararray) {
	
	var counter;
	BrainVoyager.PrintToLog(msg);
	for (counter = 0; counter < vararray.length; counter++) {
		BrainVoyager.PrintToLog(vararray[counter]);
	}	
}
/*
function printpairstolog(msg, vararray) {
	
	var counter;
	BrainVoyager.PrintToLog(msg);
	for (counter = 0; counter < vararray.length; counter++) {
		BrainVoyager.PrintToLog(vararray[counter][0] + ": " + vararray[counter][1]);
	}	
}
*/

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/* 
 iofunctions.qs  
  
 v02, 09-06-10: added functions readparametersfile() and writeparametersfile() 
*/


//  output: returns list with lines
function readeprimelogfile(filenamesfilename) {
	
	var nrfiles;
	var cancel = false;
	var logvaluesarray = new Array();	
	BrainVoyager.PrintToLog("Getting E-Prime logfile...\n");
    
    BrainVoyager.PrintToLog(filenamesfilename);
    
	if (filenamesfilename == undefined) {
		cancel = true;
	} else {	
        var headersize = 4;
		var linecounter, line;
		var reconname;
		var filenamesfile = new QFile(filenamesfilename);
		filenamesfile.open(new QIODevice.OpenMode(QIODevice.ReadOnly));
         var textstr = new QTextStream(filenamesfile); 
		while ((!textstr.atEnd()) && (!cancel)) { /// als atEnd() werkt, in scripting guide zetten!
			line = textstr.readLine();
			BrainVoyager.PrintToLog(line);
			logvaluesarray.push(line);
		}	
        filenamesfile.close();
	}
	if (cancel) BrainVoyager.PrintToLog("Further operations cancelled.");
	return logvaluesarray;	
}

function readparametersfile(parametersfilename) {
	
	var cancel = false;
	var paramsfile = new QFile(parametersfilename);
	var headersize = 6; 
	var linecounter, line, nrconditions;
	// read the header
	if (!cancel) {
        BrainVoyager.PrintToLog("\nReading parameters file...");
		paramsfile.open(new QIODevice.OpenMode(QIODevice.ReadOnly));
        var textstr = new QTextStream(paramsfile); 
		for (linecounter = 1; linecounter < headersize; linecounter++) {
			line = textstr.readLine();            
		}
	
        // read number of conditions
        line = textstr.readLine();    
        if ((line.indexOf("number of conditions:") > -1)) {
            nrconditions = parseInt(textstr.readLine());
            BrainVoyager.PrintToLog("Number of conditions: " + nrconditions);
        }
        // create experimentobject
        var expobj = new Experimentobject(nrconditions);
        var variablenamearray = new Array(nrconditions);
        var conditionnamearray = new Array(nrconditions);
        var colorindexarray = new Array(nrconditions);
        var condbeginvararray = new Array(nrconditions); 
        var condbeginproparray = new Array(nrconditions); 
        var condendvararray = new Array(nrconditions);  
        var condendproparray = new Array(nrconditions); 
        // read rest of file
        var reconname, conditionnr;
        
            line = textstr.readLine();
            if ((line.indexOf("experimentname:") > -1)) {
                var tmpline = textstr.readLine();
              //  var experimentname = tmpline;//gettargetstring(tmpline);
                var experimentname = tmpline.toString().trim();
                expobj.setexperimentname(experimentname);
            }
            // variable names
            line = textstr.readLine();
            if ((line.indexOf("variables:") > -1)) {
                for (conditionnr = 0; conditionnr < nrconditions; conditionnr++) {
                    var tmpline = textstr.readLine();
                    var variablename = tmpline.toString().trim();
                    var varname = variablename.split(" ");
                    for (i=0; i<varname.length; i++) {
                        if (varname[i].length > 0) {
                            variablename = varname[i];
                            break;
                        }
                    }
                    variablenamearray[conditionnr] = variablename;
                }
                expobj.setvariables(variablenamearray);
            }
            // condition names
            line = textstr.readLine();
            if ((line.indexOf("conditionnames:") > -1)) {
                for (conditionnr = 0; conditionnr < nrconditions; conditionnr++) {
                    var tmpline = textstr.readLine().split(" ");
                    for (i=0; i<tmpline.length; i++) {
                        if (tmpline[i].length > 0) {
                            conditionname = tmpline[i];
                            break;
                        }
                    }
                    conditionnamearray[conditionnr] = conditionname;
                }
                expobj.setconditionnames(conditionnamearray);
            }
            // color index array
            line = textstr.readLine();
            if ((line.indexOf("colors:") > -1)) {
                for (conditionnr = 0; conditionnr < nrconditions; conditionnr++) {
                    var tmpline = textstr.readLine().split(" ");
                    for (i=0; i<tmpline.length; i++) {
                        if (tmpline[i].length > 0) {
                            colorname = tmpline[i];
                            break;
                        }
                    }
                    colorindexarray[conditionnr] = getcolorindex(colorname);
                }
                expobj.setcolorindexarray(colorindexarray);	
            }
            // absolute zero
            line = textstr.readLine();
            if ((line.indexOf("zero:") > -1)) {
                var absolutezero = parseInt(textstr.readLine()); 
                expobj.setzero(absolutezero);
            }
            // begin of conditions
            line = textstr.readLine();
            if ((line.indexOf("condition begin variables:") > -1)) {
                for (conditionnr = 0; conditionnr < nrconditions; conditionnr++) {
                    var conditionbeginvar = "";
                    var tmpline = textstr.readLine().split(" ");
                    for (i=0; i<tmpline.length; i++) {
                        if (tmpline[i].length > 0) {
                            conditionbeginvar = tmpline[i];
                            break;
                        }
                    }
                    condbeginvararray[conditionnr] = conditionbeginvar;
                }
            }
            line = textstr.readLine();
            if ((line.indexOf("condition begin properties:") > -1)) {
                for (conditionnr = 0; conditionnr < nrconditions; conditionnr++) {
                    var conditionbeginprop = "";// = tmpline.toString().trim();
                    var tmpline = textstr.readLine().split(" ");
                    for (i=0; i<tmpline.length; i++) {
                        if (tmpline[i].length > 0) {
                            conditionbeginprop = tmpline[i];
                            break;
                        }
                    }
                    condbeginproparray[conditionnr] = conditionbeginprop;
                }
            }
            expobj.setconditionsbegin(condbeginvararray, condbeginproparray);
            // end of conditions
            line = textstr.readLine();
            if ((line.indexOf("condition end variables:") > -1)) {
                for (conditionnr = 0; conditionnr < nrconditions; conditionnr++) {
                    var conditionendvar = "";
                    var tmpline = textstr.readLine().split(" ");
                    for (i=0; i<tmpline.length; i++) {
                        if (tmpline[i].length > 0) {
                            conditionendvar = tmpline[i];
                            break;
                        }
                    }
                    condendvararray[conditionnr] = conditionendvar;
                }
            }
            line = textstr.readLine();
            if ((line.indexOf("condition end properties:") > -1)) {
                for (conditionnr = 0; conditionnr < nrconditions; conditionnr++) {
                    var conditionendprop = "";// = tmpline.toString().trim();
                    var tmpline = textstr.readLine().split(" ");
                    for (i=0; i<tmpline.length; i++) {
                        if (tmpline[i].length > 0) {
                            conditionendprop = tmpline[i];
                            break;
                        }
                    }
                    condendproparray[conditionnr] = conditionendprop;				
                }
            }
            expobj.setconditionsend(condendvararray, condendproparray);
        paramsfile.close();
        BrainVoyager.PrintToLog("Finished reading parameters file.");
    } else {
        BrainVoyager.PrintToLog("Further operations cancelled.");
    }
 	return expobj;	
 }

// not used in v0.5
function writeparametersfile(experimentobj, logfilename) {
	
	var today = new Date();
	var nameforfile = "_eprimer_params.txt";
	var afile = new File(logfilename);
	var dirforparamsfile = new Dir(afile.path);
	var paramsfile = new File(dirforparamsfile.filePath() + "/" + afile.baseName + nameforfile);	
	BrainVoyager.PrintToLog(nameforfile + " will be written to: " + dirforparamsfile.filePath());
	paramsfile.open(File.WriteOnly);
	var conditionnr;
	paramsfile.writeLine("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ parameters~~~~~~~~~~~~~~~~~~~~~~~~~~~");
	paramsfile.writeLine("\tCreated " + today + " by eprimer tool");
	paramsfile.writeLine("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	\n\n");
	var nrconditions = experimentobj.getnrofconditions();
	paramsfile.writeLine("number of conditions:");
	paramsfile.writeLine(nrconditions);
	paramsfile.writeLine("experimentname:");
	paramsfile.writeLine(experimentobj.getexperimentname() + " <BR>");
	paramsfile.writeLine("variables:");
	var variables = experimentobj.getvariables();
	for (conditionnr = 0; conditionnr < nrconditions; conditionnr++) {
		paramsfile.writeLine(variables[conditionnr] + " <BR>");
	}
	paramsfile.writeLine("conditionnames:");
	var conditionnamearray = experimentobj.getconditionnames();
	for (conditionnr = 0; conditionnr < nrconditions; conditionnr++) {
		paramsfile.writeLine(conditionnamearray[conditionnr] + " <BR>");
	}
	paramsfile.writeLine("colors:");
	var colorindexarray = experimentobj.getcolorindexarray();
	for (conditionnr = 0; conditionnr < nrconditions; conditionnr++) {
		paramsfile.writeLine(getcolorname(colorindexarray[conditionnr]) + " <BR>");		
	}
	paramsfile.writeLine("zero:");
	paramsfile.writeLine(experimentobj.getzero.toString());
	paramsfile.writeLine("condition begin variables:");
	var condbeginvararray = experimentobj.getconditionsvarbegin();
	for (conditionnr = 0; conditionnr < nrconditions; conditionnr++) {
		paramsfile.writeLine(condbeginvararray[conditionnr] + " <BR>");
	}
	paramsfile.writeLine("condition begin properties:");
	var condbeginproparray = experimentobj.getconditionspropbegin();
	for (conditionnr = 0; conditionnr < nrconditions; conditionnr++) {
		paramsfile.writeLine(condbeginproparray[conditionnr] + " <BR>");
	}
	paramsfile.writeLine("condition end variables:");
	var endvarlist = experimentobj.getconditionsvarend();
	for (conditionnr = 0; conditionnr < nrconditions; conditionnr++) {
		paramsfile.writeLine(endvarlist[conditionnr] + " <BR>");
	}
	paramsfile.writeLine("condition end properties:");
	var endproplist = experimentobj.getconditionspropend();
		for (conditionnr = 0; conditionnr < nrconditions; conditionnr++) {
		paramsfile.writeLine(endproplist[conditionnr] + " <BR>");
	}	
	paramsfile.close();
}


// This function is created to cope with spaces in a name and <BR> at the end of the line (obsolete)
function gettargetstring(tmpline) {

	var name, reconname;
	name = tmpline.split(' ');
	if (name.length == 2) {
		reconname = name[0];
	} else {
		var cntr; 
		reconname = "";
		for (cntr = 0; cntr < (name.length - 1); cntr++) {
			if (cntr == (name.length - 2)) {
				reconname += name[cntr];
			} else {
				reconname += name[cntr] + " ";							
			}
		}
	}	
    
	return reconname.mid(); // please note: .mid() does not work in Qt Script!!! (it does in the old QSA...)
}


// added function in v0.5
scriptObj.getFile = function(filetype) {

    var success = true;
    BrainVoyager.SetPluginStringParameter("FileType", filetype); // FMR or NIfTI
    BrainVoyager.SetPluginStringParameter("Command", "GetFileName");
    BrainVoyager.ExecutePlugin();
    var filename = BrainVoyager.GetPluginStringParameter("FileName");
    return filename;
}

// added function in v0.5
scriptObj.getLogFile = function() {

	BrainVoyager.PrintToLog("Get E-Prime Log file (*.txt)...");
    var filetype = "E-Prime Log file (*.txt)";
    var filename = this.getFile(filetype);
    if (filename.length > 0) {
        lblGetLogFile.text = filename;
        BrainVoyager.PrintToLog("Name: " + filename);
    }
}

// added function in v0.5
scriptObj.getConfigFile = function() {

	BrainVoyager.PrintToLog("Get conditions configuration file (*.txt)...");
    var filetype = "Conditions configuration file (*.txt)";
    var filename = this.getFile(filetype);
    if (filename.length > 0) {
        lblGetConfigFile.text = filename;
        BrainVoyager.PrintToLog("Name: " + filename);
    }
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// gui functions (new: BrainVoyager QX 2.3 specific)

scriptObj.initDlg = function() {

    // import widgets
    lblGetLogFile       = BrainVoyager.FindChild("lblGetLogFile");
    lblGetConfigFile    = BrainVoyager.FindChild("lblGetConfigFile");
    btnGetLogFile       = BrainVoyager.FindChild("btnGetLogFile");
    btnGetConfigFile    = BrainVoyager.FindChild("btnGetConfigFile");
    btnConvertLog2Prt   = BrainVoyager.FindChild("btnConvertLog2Prt");

   // connect widgets to script functions   
    btnGetLogFile.clicked.connect(this, this.getLogFile);
    btnGetConfigFile.clicked.connect(this, this.getConfigFile);
    btnConvertLog2Prt.clicked.connect(this, this.generate_stimulation_protocol_from_EPrimeLog); 
 
}

returnScriptObj = function()
{
   return scriptObj; // it is important to return script object containing properties and functions to be called from BrainVoyager QX
}
returnScriptObj();

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


	
